@@ -1,6 +1,6 @@
 from oct_converter.readers import E2E
 
-filepath = "/Users/mark/Downloads/Sample/001_O01S.E2E"
+filepath = "../sample_files/sample.E2E"
 file = E2E(filepath)
 oct_volumes = (
     file.read_oct_volume()
@@ -13,4 +13,4 @@ fundus_images = (
     file.read_fundus_image()
 )  # returns a list of all fundus images with additional metadata if available
 for image in fundus_images:
-    image.save("{}+{}.png".format(image.image_id, image.laterality))
\ No newline at end of file
+    image.save("{}+{}.png".format(image.image_id, image.laterality))
