@@ -1,6 +1,6 @@
 from oct_converter.readers import POCT
 
-filepath = "/Users/mark/Dropbox/Work/Projects/OCT-Converter/my_example_volumes/optovue-oct/aaa_111111_Retina Map_OD_2022-05-31_09.31.11_1.OCT"
+filepath = "../sample_files/sample.OCT"
 poct = POCT(filepath)
 oct_volumes = poct.read_oct_volume()
 
