@@ -1,70 +1,25 @@
+import io
 from pathlib import Path
 
 import numpy as np
-from construct import Int32un, PaddedString, Struct
-from pylibjpeg import decode
+from construct import ListContainer
+from PIL import Image
 
 from oct_converter.image_types import FundusImageWithMetaData, OCTVolumeWithMetaData
+from oct_converter.readers.binary_structs import fda_binary
 
 
 class FDA(object):
-    """Class for extracting data from Topcon's .fda file format.
-
-    Notes:
-        Mostly based on description of .fda file format here:
-        https://bitbucket.org/uocte/uocte/wiki/Topcon%20File%20Format
-
-    Attributes:
-        filepath (str): Path to .img file for reading.
-        header (obj:Struct): Defines structure of volume's header.
-        oct_header (obj:Struct): Defines structure of OCT header.
-        fundus_header (obj:Struct): Defines structure of fundus header.
-        chunk_dict (dict): Name of data chunks present in the file, and their start locations.
-    """
+    """Class for extracting data from Topcon's .fda file format."""
 
     def __init__(self, filepath):
         self.filepath = Path(filepath)
         if not self.filepath.exists():
             raise FileNotFoundError(self.filepath)
-        self.header = Struct(
-            "FOCT" / PaddedString(4, "ascii"),
-            "FDA" / PaddedString(3, "ascii"),
-            "version_info_1" / Int32un,
-            "version_info_2" / Int32un,
-        )
-        self.oct_header = Struct(
-            "type" / PaddedString(1, "ascii"),
-            "unknown1" / Int32un,
-            "unknown2" / Int32un,
-            "width" / Int32un,
-            "height" / Int32un,
-            "number_slices" / Int32un,
-            "unknown3" / Int32un,
-        )
-
-        self.oct_header_2 = Struct(
-            "unknown" / PaddedString(1, "ascii"),
-            "width" / Int32un,
-            "height" / Int32un,
-            "bits_per_pixel" / Int32un,
-            "number_slices" / Int32un,
-            "unknown" / PaddedString(1, "ascii"),
-            "size" / Int32un,
-        )
-
-        self.fundus_header = Struct(
-            "width" / Int32un,
-            "height" / Int32un,
-            "bits_per_pixel" / Int32un,
-            "number_slices" / Int32un,
-            "unknown" / PaddedString(4, "ascii"),
-            "size" / Int32un,
-            # 'img' / Int8un,
-        )
 
         self.chunk_dict = self.get_list_of_file_chunks()
 
-    def get_list_of_file_chunks(self):
+    def get_list_of_file_chunks(self, printing=True):
         """Find all data chunks present in the file.
 
         Returns:
@@ -74,7 +29,7 @@ class FDA(object):
         with open(self.filepath, "rb") as f:
             # skip header
             raw = f.read(15)
-            header = self.header.parse(raw)
+            header = fda_binary.header.parse(raw)
 
             eof = False
             while not eof:
@@ -87,9 +42,11 @@ class FDA(object):
                     chunk_location = f.tell()
                     f.seek(chunk_size, 1)
                     chunk_dict[chunk_name] = [chunk_location, chunk_size]
-        print("File {} contains the following chunks:".format(self.filepath))
-        for key in chunk_dict.keys():
-            print(key)
+        if printing:
+            print("File {} contains the following chunks:".format(self.filepath))
+            for key in chunk_dict.keys():
+                print(key)
+            print("")
         return chunk_dict
 
     def read_oct_volume(self):
@@ -100,20 +57,24 @@ class FDA(object):
         """
 
         if b"@IMG_JPEG" not in self.chunk_dict:
-            raise ValueError("Could not find OCT header @IMG_JPEG in chunk list")
+            print("@IMG_JPEG is not in chunk list, skipping.")
+            return None
         with open(self.filepath, "rb") as f:
             chunk_location, chunk_size = self.chunk_dict[b"@IMG_JPEG"]
             f.seek(chunk_location)  # Set the chunk’s current position.
             raw = f.read(25)
-            oct_header = self.oct_header.parse(raw)
+            oct_header = fda_binary.oct_header.parse(raw)
             volume = np.zeros(
                 (oct_header.height, oct_header.width, oct_header.number_slices)
             )
+
             for i in range(oct_header.number_slices):
                 size = np.fromstring(f.read(4), dtype=np.int32)[0]
                 raw_slice = f.read(size)
-                slice = decode(raw_slice)
+                image = Image.open(io.BytesIO(raw_slice))
+                slice = np.asarray(image)
                 volume[:, :, i] = slice
+
         oct_volume = OCTVolumeWithMetaData(
             [volume[:, :, i] for i in range(volume.shape[2])]
         )
@@ -127,12 +88,13 @@ class FDA(object):
         """
 
         if b"@IMG_MOT_COMP_03" not in self.chunk_dict:
-            raise ValueError("Could not find OCT header @IMG_MOT_COMP_03 in chunk list")
+            print("@IMG_MOT_COMP_03 is not in chunk list, skipping.")
+            return None
         with open(self.filepath, "rb") as f:
             chunk_location, chunk_size = self.chunk_dict[b"@IMG_MOT_COMP_03"]
             f.seek(chunk_location)  # Set the chunk’s current position.
             raw = f.read(22)
-            oct_header = self.oct_header_2.parse(raw)
+            oct_header = fda_binary.oct_header_2.parse(raw)
             number_pixels = (
                 oct_header.width * oct_header.height * oct_header.number_slices
             )
@@ -154,14 +116,63 @@ class FDA(object):
             obj:FundusImageWithMetaData
         """
         if b"@IMG_FUNDUS" not in self.chunk_dict:
-            raise ValueError("Could not find fundus header @IMG_FUNDUS in chunk list")
+            print("@IMG_FUNDUS is not in chunk list, skipping.")
+            return None
         with open(self.filepath, "rb") as f:
             chunk_location, chunk_size = self.chunk_dict[b"@IMG_FUNDUS"]
             f.seek(chunk_location)  # Set the chunk’s current position.
             raw = f.read(24)  # skip 24 is important
-            fundus_header = self.fundus_header.parse(raw)
+            fundus_header = fda_binary.fundus_header.parse(raw)
             number_pixels = fundus_header.width * fundus_header.height * 3
             raw_image = f.read(fundus_header.size)
-            image = decode(raw_image)
+            image = Image.open(io.BytesIO(raw_image))
+            image = np.asarray(image)
         fundus_image = FundusImageWithMetaData(image)
         return fundus_image
+
+    def read_fundus_image_gray_scale(self):
+        """Reads gray scale fundus image.
+
+        Returns:
+            obj:FundusImageWithMetaData
+        """
+        if b"@IMG_TRC_02" not in self.chunk_dict:
+            print("@IMG_TRC_02 is not in chunk list, skipping.")
+            return None
+        with open(self.filepath, "rb") as f:
+            chunk_location, chunk_size = self.chunk_dict[b"@IMG_TRC_02"]
+            f.seek(chunk_location)  # Set the chunk’s current position.
+            raw = f.read(21)  # skip 21 is important
+            img_trc_02_header = fda_binary.img_trc_02_header.parse(raw)
+            number_pixels = img_trc_02_header.width * img_trc_02_header.height * 1
+            raw_image = f.read(img_trc_02_header.size)
+            image = Image.open(io.BytesIO(raw_image))
+            image = np.asarray(image)
+        fundus_gray_scale_image = FundusImageWithMetaData(image)
+        return fundus_gray_scale_image
+
+    def read_any_info_and_make_dict(self, chunk_name):
+        """
+        Reads chunks, get data and make dictionary
+        :param chunk_name: name of the chunk which data will be taken.
+        Returns:
+            dict:Chunk info Data
+        """
+        if chunk_name not in self.chunk_dict:
+            print(f"{chunk_name} is not in chunk list, skipping.")
+            return None
+        with open(self.filepath, "rb") as f:
+            chunk_location, chunk_size = self.chunk_dict[chunk_name]
+            f.seek(chunk_location)  # Set the chunk’s current position.
+            raw = f.read()
+            header_name = f"{chunk_name.decode().split('@')[-1].lower()}_header"
+            chunk_info_header = dict(fda_binary.__dict__[header_name].parse(raw))
+            chunks_info = dict()
+            for idx, key in enumerate(chunk_info_header.keys()):
+                if idx == 0:
+                    continue
+                if type(chunk_info_header[key]) is ListContainer:
+                    chunks_info[key] = list(chunk_info_header[key])
+                else:
+                    chunks_info[key] = chunk_info_header[key]
+        return chunks_info
